@@ -1,6 +1,6 @@
 /*
- * sar, sadc, mpstat and iostat common routines.
- * (C) 1999-2004 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * sar, sadc, sadf, mpstat and iostat common routines.
+ * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -15,7 +15,7 @@
  *                                                                         *
  * You should have received a copy of the GNU General Public License along *
  * with this program; if not, write to the Free Software Foundation, Inc., *
- * 675 Mass Ave, Cambridge, MA 02139, USA.                                 *
+ * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA                   *
  ***************************************************************************
  */
 
@@ -37,7 +37,9 @@
  */
 #include <asm/page.h>
 
+#include "version.h"
 #include "common.h"
+#include "ioconf.h"
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -48,6 +50,19 @@
 #endif
 
 
+/*
+ ***************************************************************************
+ * Print sysstat version number and exit
+ ***************************************************************************
+ */
+void print_version(void)
+{
+   fprintf(stderr, _("sysstat version %s\n"
+		   "(C) Sebastien Godard\n"), VERSION);
+   exit(1);
+}
+
+
 /*
  ***************************************************************************
  * Get current date or time
@@ -68,59 +83,43 @@ time_t get_localtime(struct tm *loc_time)
 
 /*
  ***************************************************************************
- * Find number of processors used on the machine
- * (0 means one proc, 1 means two proc, etc.)
+ * Return the number of processors used on the machine:
+ * 0: one proc and non SMP kernel
+ * 1: one proc and SMP kernel, or SMP machine with all but one offlined CPU
+ * 2: two proc...
  * As far as I know, there are two possibilities for this:
- * 1) Use /proc/stat or 2) Use /proc/cpuinfo
- * (I haven't heard of a better method to guess it...)
- * See kernel sources:
- * 2.4: linux/fs/proc/proc_misc.c: kstat_read_proc()
- * 2.6: linux/fs/proc/proc_misc.c: show_stat()
+ * 1) Use /proc/stat (this is what we are doing here) or
+ * 2) Use /proc/cpuinfo
  ***************************************************************************
  */
-int get_cpu_nr(int *cpu_nr, unsigned int max_nr_cpus)
+int get_cpu_nr(unsigned int max_nr_cpus)
 {
-   FILE *statfp;
+   FILE *fp;
    char line[16];
-   int proc_nb, smp_box;
+   int proc_nb, cpu_nr = -1;
 
-   *cpu_nr = -1;
-
-   /* Open stat file */
-   if ((statfp = fopen(STAT, "r")) == NULL) {
+   if ((fp = fopen(STAT, "r")) == NULL) {
       fprintf(stderr, _("Cannot open %s: %s\n"), STAT, strerror(errno));
       exit(1);
    }
 
-   while (fgets(line, 16, statfp) != NULL) {
+   while (fgets(line, 16, fp) != NULL) {
 
       if (strncmp(line, "cpu ", 4) && !strncmp(line, "cpu", 3)) {
 	 sscanf(line + 3, "%d", &proc_nb);
-	 if (proc_nb > *cpu_nr)
-	   *cpu_nr = proc_nb;
+	 if (proc_nb > cpu_nr)
+	    cpu_nr = proc_nb;
       }
    }
 
-   /*
-    * cpu_nr initial value: -1
-    * If cpu_nr < 0 then there is only one proc.
-    * If cpu_nr > 0 then this is an SMP machine.
-    * If cpu_nr = 0 then there is only one proc but this is a Linux 2.2 SMP or
-    * Linux 2.4 kernel.
-    */
-   smp_box = (*cpu_nr > 0);
-   if (*cpu_nr < 0)
-      *cpu_nr = 0;
-
-   if (*cpu_nr >= max_nr_cpus) {
+   if ((cpu_nr + 1) > max_nr_cpus) {
       fprintf(stderr, _("Cannot handle so many processors!\n"));
       exit(1);
    }
 
-   /* Close file */
-   fclose(statfp);
+   fclose(fp);
 
-   return smp_box;
+   return (cpu_nr + 1);
 }
 
 
@@ -155,7 +154,7 @@ int get_dev_part_nr(char *dev_name)
    }
 
    /* Close directory */
-   (void) closedir(dir);
+   closedir(dir);
 
    return part;
 }
@@ -168,7 +167,7 @@ int get_dev_part_nr(char *dev_name)
  * directory), then find number of devices registered.
  ***************************************************************************
  */
-int get_sysfs_dev_nr(int flags)
+int get_sysfs_dev_nr(int display_partitions)
 {
    DIR *dir;
    struct dirent *drd;
@@ -191,14 +190,14 @@ int get_sysfs_dev_nr(int flags)
 	 /* Yep... */
 	 dev++;
 	
-	 if (DISPLAY_PARTITIONS(flags))
+	 if (display_partitions)
 	    /* We also want the number of partitions for this device */
 	    dev += get_dev_part_nr(drd->d_name);
       }
    }
 
     /* Close /sys/block directory */
-   (void) closedir(dir);
+   closedir(dir);
 
    return dev;
 }
@@ -206,40 +205,43 @@ int get_sysfs_dev_nr(int flags)
 
 /*
  ***************************************************************************
- * Find number of devices and partitions available in /proc/diskstats
- * See kernel sources:
- * 2.6: linux/drivers/block/genhd.c: diskstats_show()
+ * Find number of devices and partitions available in /proc/diskstats.
+ * Args: count_part : set to TRUE if devices _and_ partitions are to be
+ *		counted.
+ *       only_used_dev : when counting devices, set to TRUE if only devices
+ 		with non zero stats are to be counted.
  ***************************************************************************
  */
-int get_diskstats_dev_nr(int count_part)
+int get_diskstats_dev_nr(int count_part, int only_used_dev)
 {
-   FILE *dstatsfp;
+   FILE *fp;
    char line[256];
    int dev = 0, i;
-   unsigned int tmp[2];
+   unsigned long rd_ios, wr_ios;
 
-   /* Open /proc/diskstats file */
-   if ((dstatsfp = fopen(DISKSTATS, "r")) == NULL)
-      /* /proc/diskstats non-existent */
+   if ((fp = fopen(DISKSTATS, "r")) == NULL)
+      /* File non-existent */
       return 0;
 
    /*
     * Counting devices and partitions is simply a matter of counting
     * the number of lines...
     */
-   while (fgets(line, 256, dstatsfp) != NULL) {
+   while (fgets(line, 256, fp) != NULL) {
       if (!count_part) {
-	 i = sscanf(line, "%*d %*d %*s %*u %*u %*u %u %u",
-		    &tmp[0], &tmp[1]);
+	 i = sscanf(line, "%*d %*d %*s %lu %*u %*u %*u %lu",
+		    &rd_ios, &wr_ios);
 	 if (i == 1)
 	    /* It was a partition and not a device */
 	    continue;
+	 if (only_used_dev && !rd_ios && !wr_ios)
+	    /* Unused device */
+	    continue;
       }
       dev++;
    }
 
-   /* Close file */
-   fclose(dstatsfp);
+   fclose(fp);
 
    return dev;
 }
@@ -249,34 +251,34 @@ int get_diskstats_dev_nr(int count_part)
  ***************************************************************************
  * Find number of devices and partitions that have statistics in
  * /proc/partitions.
- * See kernel sources:
- * 2.6: linux/drivers/block/genhd.c: show_partition() (see sysfs instead)
  ***************************************************************************
  */
-int get_ppartitions_dev_nr(void)
+int get_ppartitions_dev_nr(int count_part)
 {
-   FILE *ppartfp;
+   FILE *fp;
    char line[256];
    int dev = 0;
-   unsigned int tmp;
+   unsigned int major, minor, tmp;
 
-   /* Open /proc/partitions file */
-   if ((ppartfp = fopen(PPARTITIONS, "r")) == NULL)
-      /* /proc/partitions non-existent */
+   if ((fp = fopen(PPARTITIONS, "r")) == NULL)
+      /* File non-existent */
       return 0;
 
-   while (fgets(line, 256, ppartfp) != NULL) {
-      if (sscanf(line, "%*u %*u %*u %*s %u", &tmp) == 1)
+   while (fgets(line, 256, fp) != NULL) {
+      if (sscanf(line, "%u %u %*u %*s %u", &major, &minor, &tmp) == 3) {
 	 /*
 	  * We have just read a line from /proc/partitions containing stats
-	  * for a device or a partition
-	  * (i.e. this is not a fake line: title, etc.)
+	  * for a device or a partition (i.e. this is not a fake line:
+	  * header, blank line,... or a line without stats!)
 	  */
+	 if (!count_part && !ioc_iswhole(major, minor))
+	    /* This was a partition, and we don't want to count them */
+	    continue;
 	 dev++;
+      }
    }
 
-   /* Close file */
-   fclose(ppartfp);
+   fclose(fp);
 
    return dev;
 }
@@ -290,18 +292,17 @@ int get_ppartitions_dev_nr(void)
  */
 unsigned int get_disk_io_nr(void)
 {
-   FILE *statfp;
+   FILE *fp;
    char line[8192];
    unsigned int dsk = 0;
    int pos;
 
-   /* Open /proc/stat file */
-   if ((statfp = fopen(STAT, "r")) == NULL) {
+   if ((fp = fopen(STAT, "r")) == NULL) {
       fprintf(stderr, _("Cannot open %s: %s\n"), STAT, strerror(errno));
       exit(2);
    }
 
-   while (fgets(line, 8192, statfp) != NULL) {
+   while (fgets(line, 8192, fp) != NULL) {
 
       if (!strncmp(line, "disk_io: ", 9)) {
 	 for (pos = 9; pos < strlen(line) - 1; pos +=strcspn(line + pos, " ") + 1)
@@ -309,8 +310,7 @@ unsigned int get_disk_io_nr(void)
       }
    }
 
-   /* Close /proc/stat file */
-   fclose(statfp);
+   fclose(fp);
 
    return dsk;
 }
@@ -419,6 +419,9 @@ int get_kb_shift(void)
  * counter, if @value2 < @value1 and @value1 <= 0xffffffff, then we can
  * assume that the counter's type was unsigned long and has overflown, and
  * so the difference @value2 - @value1 must be casted to this type.
+ * NOTE: These functions should no longer be necessary to handle a particular
+ * stat counter when we can assume that everybody is using a recent kernel
+ * (defining this counter as unsigned long long).
  ***************************************************************************
  */
 double ll_sp_value(unsigned long long value1, unsigned long long value2,
@@ -440,4 +443,3 @@ double ll_s_value(unsigned long long value1, unsigned long long value2,
    else
       return S_VALUE(value1, value2, itv);
 }
-
