@@ -1,6 +1,6 @@
 /*
  * sysstat: System performance tools for Linux
- * (C) 1999-2004 by Sebastien Godard (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien Godard (sysstat <at> wanadoo.fr)
  */
 
 #ifndef _COMMON_H
@@ -39,20 +39,10 @@
 #define NR_DEV_PREALLOC		4
 #define NR_DISK_PREALLOC	3
 
-/* Define flags */
-#define F_BOOT_STATS		0x100000
-#define D_PARTITIONS		0x200000
-#define F_HAS_DISKSTATS		0x400000
-#define F_HAS_PPARTITIONS	0x800000
-
-#define WANT_BOOT_STATS(m)	(((m) & F_BOOT_STATS) == F_BOOT_STATS)
-#define DISPLAY_PARTITIONS(m)	(((m) & D_PARTITIONS) == D_PARTITIONS)
-#define HAS_DISKSTATS(m)	(((m) & F_HAS_DISKSTATS) == F_HAS_DISKSTATS)
-#define HAS_PPARTITIONS(m)	(((m) & F_HAS_PPARTITIONS) == F_HAS_PPARTITIONS)
-
-
 #define CNT_DEV		0
 #define CNT_PART	1
+#define CNT_ALL_DEV	0
+#define CNT_USED_DEV	1
 
 #define S_VALUE(m,n,p)	(((double) ((n) - (m))) / (p) * HZ)
 
@@ -82,16 +72,17 @@ extern char	   *device_name(char *);
 extern unsigned int get_disk_io_nr(void);
 extern int	    get_kb_shift(void);
 extern time_t	    get_localtime(struct tm *);
-extern int	    get_cpu_nr(int *, unsigned int);
+extern int	    get_cpu_nr(unsigned int);
 extern int	    get_sysfs_dev_nr(int);
-extern int	    get_diskstats_dev_nr(int);
-extern int	    get_ppartitions_dev_nr(void);
+extern int	    get_diskstats_dev_nr(int, int);
+extern int	    get_ppartitions_dev_nr(int);
 extern int	    get_win_height(void);
 extern void	    init_nls(void);
-extern double       ll_s_value(unsigned long long, unsigned long long,
+extern double	    ll_s_value(unsigned long long, unsigned long long,
 			       unsigned long long);
-extern double       ll_sp_value(unsigned long long, unsigned long long,
+extern double	    ll_sp_value(unsigned long long, unsigned long long,
 				unsigned long long);
 extern void	    print_gal_header(struct tm *, char *, char *, char *);
+extern void	    print_version(void);
 
 #endif  /* _COMMON_H */
