@@ -1,6 +1,6 @@
 /*
  * iostat: report CPU and I/O statistics
- * (C) 1998-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1998-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -62,11 +62,8 @@ int dlist_idx = 0;
 long interval = 0;
 unsigned char timestamp[64];
 
-/*
- * Nb of processors on the machine.
- * A value of 1 means two procs...
- */
-int cpu_nr = -1;
+/* Nb of processors on the machine */
+int cpu_nr = 0;
 
 
 /*
@@ -391,7 +388,7 @@ void read_proc_stat(int curr, int flags)
 	    			   comm_stats[curr].cpu_steal;
       }
 
-      else if ((!strncmp(line, "cpu0", 4)) && cpu_nr) {
+      else if ((!strncmp(line, "cpu0", 4)) && (cpu_nr > 1)) {
 	 /*
 	  * Read CPU line for proc#0 (if available).
 	  * Useful to compute uptime reduced to one processor on SMP machines,
@@ -797,15 +794,17 @@ void write_stat_header(int flags, int *fctr)
 {
    if (DISPLAY_EXTENDED(flags)) {
       /* Extended stats */
-      printf("Device:    rrqm/s wrqm/s   r/s   w/s  rsec/s  wsec/s");
+      printf("Device:    rrqm/s wrqm/s   r/s   w/s");
       if (DISPLAY_MEGABYTES(flags)) {
 	 printf("    rMB/s    wMB/s");
 	 *fctr = 2048;
       }
-      else {
+      else if (DISPLAY_KILOBYTES(flags)) {
 	 printf("    rkB/s    wkB/s");
 	 *fctr = 2;
       }
+      else
+	 printf("   rsec/s   wsec/s");
       printf(" avgrq-sz avgqu-sz   await  svctm  %%util\n");
    }
    else {
@@ -847,14 +846,14 @@ void write_ext_stat(int curr, unsigned long long itv, int flags, int fctr,
     * sometimes negative...
     */
    nr_ios = (ioi->rd_ios - ioj->rd_ios) + (ioi->wr_ios - ioj->wr_ios);
-   tput   = ((double) nr_ios) * HZ / itv;
+   tput = ((double) nr_ios) * HZ / itv;
    util = S_VALUE(ioj->tot_ticks, ioi->tot_ticks, itv);
-   svctm  = tput ? util / tput : 0.0;
+   svctm = tput ? util / tput : 0.0;
    /*
     * kernel gives ticks already in milliseconds for all platforms
     * => no need for further scaling.
     */
-   await  = nr_ios ?
+   await = nr_ios ?
       ((ioi->rd_ticks - ioj->rd_ticks) + (ioi->wr_ticks - ioj->wr_ticks)) /
       nr_ios : 0.0;
 
@@ -865,19 +864,17 @@ void write_ext_stat(int curr, unsigned long long itv, int flags, int fctr,
    if ((ioi->wr_sectors < ioj->wr_sectors) && (ioj->wr_sectors <= 0xffffffff))
       wr_sec &= 0xffffffff;
 
-   arqsz  = nr_ios ? (rd_sec + wr_sec) / nr_ios : 0.0;
+   arqsz = nr_ios ? (rd_sec + wr_sec) / nr_ios : 0.0;
 
    printf("%-10s", shi->name);
    if (strlen(shi->name) > 10)
       printf("\n          ");
-   /*       rrq/s wrq/s   r/s   w/s  rsec  wsec   r?B   w?B  rqsz  qusz await svctm %util */
-   printf(" %6.2f %6.2f %5.2f %5.2f %7.2f %7.2f %8.2f %8.2f %8.2f %8.2f %7.2f %6.2f %6.2f\n",
+   /*       rrq/s wrq/s   r/s   w/s  rsec  wsec  rqsz  qusz await svctm %util */
+   printf(" %6.2f %6.2f %5.2f %5.2f %8.2f %8.2f %8.2f %8.2f %7.2f %6.2f %6.2f\n",
 	  S_VALUE(ioj->rd_merges, ioi->rd_merges, itv),
 	  S_VALUE(ioj->wr_merges, ioi->wr_merges, itv),
 	  S_VALUE(ioj->rd_ios, ioi->rd_ios, itv),
 	  S_VALUE(ioj->wr_ios, ioi->wr_ios, itv),
-	  ll_s_value(ioj->rd_sectors, ioi->rd_sectors, itv),
-	  ll_s_value(ioj->wr_sectors, ioi->wr_sectors, itv),
 	  ll_s_value(ioj->rd_sectors, ioi->rd_sectors, itv) / fctr,
 	  ll_s_value(ioj->wr_sectors, ioi->wr_sectors, itv) / fctr,
 	  arqsz,
@@ -988,7 +985,7 @@ int write_stat(int curr, int flags, struct tm *loc_time)
       /* Display CPU utilization */
       write_cpu_stat(curr, itv);
 
-   if (cpu_nr) {
+   if (cpu_nr > 1) {
       /* On SMP machines, reduce itv to one processor (see note above) */
       if (!comm_stats[!curr].uptime0)
 	 itv = comm_stats[curr].uptime0;
