@@ -1,6 +1,6 @@
 /*
  * sar/sadc: report system activity
- * (C) 1999-2004 by Sebastien Godard (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien Godard (sysstat <at> wanadoo.fr)
  */
 
 #ifndef _SA_H
@@ -14,6 +14,13 @@
 
 #include "common.h"
 
+/*
+ * System activity daily file magic number
+ * (will vary when file format changes)
+ */
+#define SA_MAGIC	0x2169
+
+
 /* Define activities */
 #define A_PROC		0x000001
 #define A_CTXSW		0x000002
@@ -29,7 +36,8 @@
 #define A_DISK		0x000800
 #define A_PID		0x001000
 #define A_CPID		0x002000
-/* 4000-8000: unused */
+#define A_NET_NFS	0x004000
+#define A_NET_NFSD	0x008000
 #define A_PAGE		0x010000
 #define A_MEM_AMT	0x020000
 #define A_KTABLES	0x040000
@@ -57,6 +65,8 @@
 #define GET_NET_DEV(m)	(((m) & A_NET_DEV) == A_NET_DEV)
 #define GET_NET_EDEV(m)	(((m) & A_NET_EDEV) == A_NET_EDEV)
 #define GET_NET_SOCK(m)	(((m) & A_NET_SOCK) == A_NET_SOCK)
+#define GET_NET_NFS(m)	(((m) & A_NET_NFS) == A_NET_NFS)
+#define GET_NET_NFSD(m)	(((m) & A_NET_NFSD) == A_NET_NFSD)
 #define GET_QUEUE(m)	(((m) & A_QUEUE) == A_QUEUE)
 #define GET_DISK(m)	(((m) & A_DISK) == A_DISK)
 
@@ -74,40 +84,46 @@
 #define K_SADC	"SADC"
 #define K_DEV	"DEV"
 #define K_EDEV	"EDEV"
+#define K_NFS	"NFS"
+#define K_NFSD	"NFSD"
 #define K_SOCK	"SOCK"
-#define K_FULL	"FULL"
 
-/* Define flags */
-#define F_ALL_PROC      0x0001
-#define F_SA_ROTAT      0x0002
-#define F_FLT_INC	0x0004
-#define F_A_OPTION	0x0008
-#define F_F_OPTION	0x0010
-#define F_H_OPTION	0x0020
-#define F_ORG_TIME	0x0040
-#define F_DEFAULT_COUNT	0x0080
-#define F_I_OPTION	0x0100
-#define F_DB_OPTION	0x0200
-#define F_DO_SA_ROTAT	0x0400
-#define F_PER_PROC	0x0800
-#define F_L_OPTION	0x1000
-#define F_FILE_LCK	0X2000
-/* 0x100000:0x800000 -> reserved (cf. common.h) */
+/* S_= sar/sadc/sadf - F_= Flag */
+#define S_F_ALL_PROC      	0x0001
+#define S_F_SA_ROTAT      	0x0002
+#define S_F_DEV_PRETTY		0x0004
+#define S_F_A_OPTION		0x0008
+#define S_F_F_OPTION		0x0010
+#define S_F_I_OPTION		0x0020
+#define S_F_TRUE_TIME		0x0040
+#define S_F_L_OPTION		0x0080
+#define S_F_HAS_PPARTITIONS	0x0100
+#define S_F_HAS_DISKSTATS	0x0200
+#define S_F_FILE_LCK		0X0400
+#define S_F_PER_PROC		0x0800
+#define S_F_X_ALL		0x1000
 
-#define WANT_ALL_PROC(m)	(((m) & F_ALL_PROC) == F_ALL_PROC)
-#define WANT_SA_ROTAT(m)	(((m) & F_SA_ROTAT) == F_SA_ROTAT)
-#define FLT_ARE_INC(m)		(((m) & F_FLT_INC) == F_FLT_INC)
-#define USE_A_OPTION(m)		(((m) & F_A_OPTION) == F_A_OPTION)
-#define USE_F_OPTION(m)		(((m) & F_F_OPTION) == F_F_OPTION)
-#define USE_H_OPTION(m)		(((m) & F_H_OPTION) == F_H_OPTION)
-#define PRINT_ORG_TIME(m)	(((m) & F_ORG_TIME) == F_ORG_TIME)
-#define USE_DEFAULT_COUNT(m)	(((m) & F_DEFAULT_COUNT) == F_DEFAULT_COUNT)
-#define USE_I_OPTION(m)		(((m) & F_I_OPTION) == F_I_OPTION)
-#define USE_DB_OPTION(m)	(((m) & F_DB_OPTION) == F_DB_OPTION)
-#define DO_SA_ROTAT(m)		(((m) & F_DO_SA_ROTAT) == F_DO_SA_ROTAT)
-#define WANT_PER_PROC(m)	(((m) & F_PER_PROC) == F_PER_PROC)
-#define USE_L_OPTION(m)		(((m) & F_L_OPTION) == F_L_OPTION)
-#define FILE_LOCKED(m)		(((m) & F_FILE_LCK) == F_FILE_LCK)
+#define WANT_ALL_PROC(m)	(((m) & S_F_ALL_PROC) == S_F_ALL_PROC)
+#define WANT_SA_ROTAT(m)	(((m) & S_F_SA_ROTAT) == S_F_SA_ROTAT)
+#define USE_PRETTY_OPTION(m)	(((m) & S_F_DEV_PRETTY) == S_F_DEV_PRETTY)
+#define USE_A_OPTION(m)		(((m) & S_F_A_OPTION) == S_F_A_OPTION)
+#define USE_F_OPTION(m)		(((m) & S_F_F_OPTION) == S_F_F_OPTION)
+#define USE_I_OPTION(m)		(((m) & S_F_I_OPTION) == S_F_I_OPTION)
+#define PRINT_TRUE_TIME(m)	(((m) & S_F_TRUE_TIME) == S_F_TRUE_TIME)
+#define USE_L_OPTION(m)		(((m) & S_F_L_OPTION) == S_F_L_OPTION)
+#define HAS_PPARTITIONS(m)	(((m) & S_F_HAS_PPARTITIONS) == S_F_HAS_PPARTITIONS)
+#define HAS_DISKSTATS(m)	(((m) & S_F_HAS_DISKSTATS) == S_F_HAS_DISKSTATS)
+#define FILE_LOCKED(m)		(((m) & S_F_FILE_LCK) == S_F_FILE_LCK)
+#define WANT_PER_PROC(m)	(((m) & S_F_PER_PROC) == S_F_PER_PROC)
+#define WANT_ALL_PIDS(m)	(((m) & S_F_X_ALL) == S_F_X_ALL)
+
+/* Output formats (O_= Output)  */
+#define S_O_NONE		0
+#define S_O_HDR_OPTION		1
+#define S_O_PPC_OPTION		2
+#define S_O_DB_OPTION		3
+#define S_O_XML_OPTION		4
+#define S_O_DBD_OPTION		5
 
 /* Files */
 #define PROC		"/proc"
@@ -126,20 +142,21 @@
 #define FRTSIG_MAX	"/proc/sys/kernel/rtsig-max"
 #define NET_DEV		"/proc/net/dev"
 #define NET_SOCKSTAT	"/proc/net/sockstat"
+#define NET_RPC_NFS	"/proc/net/rpc/nfs"
+#define NET_RPC_NFSD	"/proc/net/rpc/nfsd"
 #define SADC		"sadc"
-#define SADC_PATH	"/usr/lib/sa/sadc"
-#define SADC_LOCAL_PATH	"/usr/local/lib/sa/sadc"
 #define LOADAVG		"/proc/loadavg"
 #define VMSTAT		"/proc/vmstat"
 
-#define NR_IRQS		256
-
 #define NR_IFACE_PREALLOC	2
 #define NR_SERIAL_PREALLOC	2
 #define NR_IRQPROC_PREALLOC	3
 
+#define NR_IRQS			256
+
 /* Maximum number of processes that can be monitored simultaneously */
 #define MAX_PID_NR	256
+
 /* Maximum length of network interface name */
 #define MAX_IFACE_LEN	IFNAMSIZ
 /*
@@ -158,19 +175,20 @@
 #define NO_RESET	0
 #define NON_FATAL	0
 #define FATAL		1
+#define C_SAR		0
+#define C_SADF		1
+
+#define X_PID_SET	0x01
+#define X_PPID_SET	0x02
 
 /* Record type */
 #define R_STATS		1
 #define R_DUMMY		2
+#define R_LAST_STATS	3
 
 #define SOFT_SIZE	0
 #define HARD_SIZE	1
 
-/*
- * System activity daily file magic number
- * (will vary when file format changes)
- */
-#define SA_MAGIC	0x2163
 
 /*
  * IMPORTANT NOTE:
@@ -186,85 +204,88 @@
  * can be safely used.
  *
  * Structures are padded so that their length be a multiple of 8 bytes.
- * It seems better (although not compulsory) for structures written
+ * It is better (although not compulsory) for structures written
  * contiguously into daily data files and accessed the following way once
  * they are read into memory:
  * struct *foo2 = struct *foo1 + i;  (since i <=> sizeof(struct foo))
  */
+
 /* System activity data file header */
 struct file_hdr {
-   /* Activity flag */
-   unsigned int	 sa_actflag			__attribute__ ((aligned (8)));
-   /* System activity data file magic number */
-   short    int  sa_magic			__attribute__ ((packed));
-   /* file_stats structure size */
-   short    int  sa_st_size			__attribute__ ((packed));
-   /* Number of processes to monitor ( {-x | -X } ALL) */
-   unsigned int  sa_nr_pid			__attribute__ ((packed));
-   /* Number of interrupts per procesor: 2 means two interrupts */
-   unsigned int  sa_irqcpu			__attribute__ ((packed));
+   /* --- LONG --- */
    /* Time stamp in seconds since the epoch */
-   unsigned long sa_ust_time			__attribute__ ((aligned (8)));
+   unsigned long  sa_ust_time			__attribute__ ((aligned (8)));
+   /* --- INT --- */
+   /* Activity flag */
+   unsigned int	  sa_actflag			__attribute__ ((aligned (8)));
+   /* Number of processes to monitor ( {-x | -X } ALL) */
+   unsigned int   sa_nr_pid			__attribute__ ((packed));
+   /* Number of interrupts per processor: 2 means two interrupts */
+   unsigned int   sa_irqcpu			__attribute__ ((packed));
    /* Number of disks */
-   unsigned int  sa_nr_disk			__attribute__ ((aligned (8)));
-   /* Number of processors: 1 means two proc */
-   unsigned int sa_proc 			__attribute__ ((packed));
+   unsigned int   sa_nr_disk			__attribute__ ((packed));
+   /* Number of processors:
+    * 0 means 1 proc and non SMP machine
+    * 1 means 1 proc and SMP machine
+    * 2 means two proc, etc.
+    */
+   unsigned int   sa_proc 			__attribute__ ((packed));
    /* Number of serial lines: 2 means two lines (ttyS00 and ttyS01) */
-   unsigned int sa_serial 			__attribute__ ((packed));
+   unsigned int   sa_serial 			__attribute__ ((packed));
    /* Number of network devices (interfaces): 2 means two lines */
-   unsigned int sa_iface 			__attribute__ ((packed));
+   unsigned int   sa_iface 			__attribute__ ((packed));
+   /* --- SHORT --- */
+   /* System activity data file magic number */
+   unsigned short sa_magic			__attribute__ ((packed));
+   /* file_stats structure size */
+   unsigned short sa_st_size			__attribute__ ((packed));
+   /* --- CHAR --- */
    /*
     * Current day, month and year.
     * No need to save DST (daylight saving time) flag, since it is not taken
     * into account by the strftime() function used to print the timestamp.
     */
-   unsigned char sa_day				__attribute__ ((packed));
-   unsigned char sa_month			__attribute__ ((packed));
-   unsigned char sa_year			__attribute__ ((packed));
+   unsigned char  sa_day			__attribute__ ((packed));
+   unsigned char  sa_month			__attribute__ ((packed));
+   unsigned char  sa_year			__attribute__ ((packed));
+   /*
+    * Size of a long integer. Useful to know the architecture on which
+    * the datafile was created.
+    */
+   char		  sa_sizeof_long		__attribute__ ((packed));
    /* Operating system name */
-   char          sa_sysname[UTSNAME_LEN]	__attribute__ ((packed));
+   char           sa_sysname[UTSNAME_LEN]	__attribute__ ((packed));
    /* Machine hostname */
-   char          sa_nodename[UTSNAME_LEN]	__attribute__ ((packed));
+   char           sa_nodename[UTSNAME_LEN]	__attribute__ ((packed));
    /* Operating system release number */
-   char          sa_release[UTSNAME_LEN]	__attribute__ ((packed));
+   char           sa_release[UTSNAME_LEN]	__attribute__ ((packed));
 };
 
 #define FILE_HDR_SIZE	(sizeof(struct file_hdr))
 
 struct file_stats {
-   /* Record type: R_STATS or R_DUMMY */
-   unsigned char record_type			__attribute__ ((aligned (8)));
-   /* Time stamp: hour, minute and second */
-   unsigned char hour		/* (0-23) */	__attribute__ ((packed));
-   unsigned char minute		/* (0-59) */	__attribute__ ((packed));
-   unsigned char second		/* (0-59) */	__attribute__ ((packed));
-   /* Nb of processes (set only when using '-x SUM') */
-   unsigned int  nr_processes			__attribute__ ((packed));
-   /* Time stamp (number of seconds since the epoch) */
-   unsigned long ust_time			__attribute__ ((aligned (8)));
+   /* --- LONG LONG --- */
    /* Machine uptime (multiplied by the # of proc) */
-   unsigned long uptime				__attribute__ ((aligned (8)));
+   unsigned long long uptime			__attribute__ ((aligned (16)));
    /* Uptime reduced to one processor. Set *only* on SMP machines */
-   unsigned long uptime0			__attribute__ ((aligned (8)));
-   /* Stats... */
+   unsigned long long uptime0			__attribute__ ((aligned (16)));
+   unsigned long long context_swtch		__attribute__ ((aligned (16)));
+   unsigned long long cpu_user			__attribute__ ((aligned (16)));
+   unsigned long long cpu_nice			__attribute__ ((aligned (16)));
+   unsigned long long cpu_system		__attribute__ ((aligned (16)));
+   unsigned long long cpu_idle			__attribute__ ((aligned (16)));
+   unsigned long long cpu_iowait		__attribute__ ((aligned (16)));
+   unsigned long long cpu_steal			__attribute__ ((aligned (16)));
+   unsigned long long irq_sum			__attribute__ ((aligned (16)));
+   /* --- LONG --- */
+   /* Time stamp (number of seconds since the epoch) */
+   unsigned long ust_time			__attribute__ ((aligned (16)));
    unsigned long processes			__attribute__ ((aligned (8)));
-   unsigned int  context_swtch			__attribute__ ((aligned (8)));
-   unsigned int  cpu_user			__attribute__ ((packed));
-   unsigned int  cpu_nice			__attribute__ ((packed));
-   unsigned int  cpu_system			__attribute__ ((packed));
-   unsigned long cpu_idle			__attribute__ ((aligned (8)));
-   unsigned long cpu_iowait			__attribute__ ((aligned (8)));
-   unsigned long irq_sum			__attribute__ ((aligned (8)));
    unsigned long pgpgin				__attribute__ ((aligned (8)));
    unsigned long pgpgout			__attribute__ ((aligned (8)));
    unsigned long pswpin				__attribute__ ((aligned (8)));
    unsigned long pswpout			__attribute__ ((aligned (8)));
-   unsigned int  dk_drive			__attribute__ ((aligned (8)));
-   unsigned int  dk_drive_rio			__attribute__ ((packed));
-   unsigned int  dk_drive_wio			__attribute__ ((packed));
-   unsigned int  dk_drive_rblk			__attribute__ ((packed));
-   unsigned int  dk_drive_wblk			__attribute__ ((packed));
-   /* Memory stats in Kb */
+   /* Memory stats in kB */
    unsigned long frmkb				__attribute__ ((aligned (8)));
    unsigned long bufkb				__attribute__ ((aligned (8)));
    unsigned long camkb				__attribute__ ((aligned (8)));
@@ -272,7 +293,16 @@ struct file_stats {
    unsigned long frskb				__attribute__ ((aligned (8)));
    unsigned long tlskb				__attribute__ ((aligned (8)));
    unsigned long caskb				__attribute__ ((aligned (8)));
-   unsigned int  file_used			__attribute__ ((aligned (8)));
+   unsigned long nr_running			__attribute__ ((aligned (8)));
+   unsigned long pgfault			__attribute__ ((aligned (8)));
+   unsigned long pgmajfault			__attribute__ ((aligned (8)));
+   /* --- INT --- */
+   unsigned int  dk_drive			__attribute__ ((aligned (8)));
+   unsigned int  dk_drive_rio			__attribute__ ((packed));
+   unsigned int  dk_drive_wio			__attribute__ ((packed));
+   unsigned int  dk_drive_rblk			__attribute__ ((packed));
+   unsigned int  dk_drive_wblk			__attribute__ ((packed));
+   unsigned int  file_used			__attribute__ ((packed));
    unsigned int  inode_used			__attribute__ ((packed));
    unsigned int  super_used			__attribute__ ((packed));
    unsigned int  super_max			__attribute__ ((packed));
@@ -285,51 +315,74 @@ struct file_stats {
    unsigned int  udp_inuse			__attribute__ ((packed));
    unsigned int  raw_inuse			__attribute__ ((packed));
    unsigned int  frag_inuse			__attribute__ ((packed));
-   unsigned long pgfault			__attribute__ ((aligned (8)));
-   unsigned long pgmajfault			__attribute__ ((aligned (8)));
-   unsigned int  dentry_stat			__attribute__ ((aligned (8)));
+   unsigned int  dentry_stat			__attribute__ ((packed));
    unsigned int  load_avg_1			__attribute__ ((packed));
    unsigned int  load_avg_5			__attribute__ ((packed));
    unsigned int  load_avg_15			__attribute__ ((packed));
-   unsigned int  nr_running			__attribute__ ((packed));
    unsigned int  nr_threads			__attribute__ ((packed));
+   unsigned int  nfs_rpccnt			__attribute__ ((packed));
+   unsigned int  nfs_rpcretrans			__attribute__ ((packed));
+   unsigned int  nfs_readcnt			__attribute__ ((packed));
+   unsigned int  nfs_writecnt			__attribute__ ((packed));
+   unsigned int  nfs_accesscnt			__attribute__ ((packed));
+   unsigned int  nfs_getattcnt			__attribute__ ((packed));
+   unsigned int  nfsd_rpccnt			__attribute__ ((packed));
+   unsigned int  nfsd_rpcbad			__attribute__ ((packed));
+   unsigned int  nfsd_netcnt			__attribute__ ((packed));
+   unsigned int  nfsd_netudpcnt			__attribute__ ((packed));
+   unsigned int  nfsd_nettcpcnt			__attribute__ ((packed));
+   unsigned int  nfsd_rchits			__attribute__ ((packed));
+   unsigned int  nfsd_rcmisses			__attribute__ ((packed));
+   unsigned int  nfsd_readcnt			__attribute__ ((packed));
+   unsigned int  nfsd_writecnt			__attribute__ ((packed));
+   unsigned int  nfsd_accesscnt			__attribute__ ((packed));
+   unsigned int  nfsd_getattcnt			__attribute__ ((packed));
+   /* --- CHAR --- */
+   /* Record type: R_STATS or R_DUMMY */
+   unsigned char record_type			__attribute__ ((packed));
+   /*
+    * Time stamp: hour, minute and second.
+    * Used to determine TRUE time (immutable, non locale dependent time).
+    */
+   unsigned char hour		/* (0-23) */	__attribute__ ((packed));
+   unsigned char minute		/* (0-59) */	__attribute__ ((packed));
+   unsigned char second		/* (0-59) */	__attribute__ ((packed));
 };
 
 #define FILE_STATS_SIZE	(sizeof(struct file_stats))
 
 struct stats_one_cpu {
-   unsigned long per_cpu_idle			__attribute__ ((aligned (8)));
-   unsigned long per_cpu_iowait			__attribute__ ((aligned (8)));
-   unsigned int  per_cpu_user			__attribute__ ((aligned (8)));
-   unsigned int  per_cpu_nice			__attribute__ ((packed));
-   unsigned int  per_cpu_system			__attribute__ ((packed));
-   unsigned char pad[4]				__attribute__ ((packed));
-/*
- * IMPORTANT NOTE:
- * Structure must be a multiple of 8 bytes, since we use an array of structures.
- * Each structure is *aligned*, and we want the structures to be packed together.
- */
+   unsigned long long per_cpu_idle		__attribute__ ((aligned (16)));
+   unsigned long long per_cpu_iowait		__attribute__ ((aligned (16)));
+   unsigned long long per_cpu_user		__attribute__ ((aligned (16)));
+   unsigned long long per_cpu_nice		__attribute__ ((aligned (16)));
+   unsigned long long per_cpu_system		__attribute__ ((aligned (16)));
+   unsigned long long per_cpu_steal		__attribute__ ((aligned (16)));
+   unsigned long long pad			__attribute__ ((aligned (16)));
 };
 
 #define STATS_ONE_CPU_SIZE	(sizeof(struct stats_one_cpu))
 
+/*
+ * Members do not need to be aligned since these stats are not written
+ * to daily data files.
+ */
 struct pid_stats {
    /* If pid is null, the process has been killed */
    unsigned long pid				__attribute__ ((aligned (8)));
-   unsigned long minflt				__attribute__ ((aligned (8)));
-   unsigned long majflt				__attribute__ ((aligned (8)));
-   unsigned long utime				__attribute__ ((aligned (8)));
-   unsigned long stime				__attribute__ ((aligned (8)));
-   unsigned long nswap				__attribute__ ((aligned (8)));
-   unsigned long cminflt			__attribute__ ((aligned (8)));
-   unsigned long cmajflt			__attribute__ ((aligned (8)));
-   unsigned long cutime				__attribute__ ((aligned (8)));
-   unsigned long cstime				__attribute__ ((aligned (8)));
-   unsigned long cnswap				__attribute__ ((aligned (8)));
-   unsigned int  processor			__attribute__ ((aligned (8)));
+   unsigned long minflt				__attribute__ ((packed));
+   unsigned long majflt				__attribute__ ((packed));
+   unsigned long utime				__attribute__ ((packed));
+   unsigned long stime				__attribute__ ((packed));
+   unsigned long nswap				__attribute__ ((packed));
+   unsigned long cminflt			__attribute__ ((packed));
+   unsigned long cmajflt			__attribute__ ((packed));
+   unsigned long cutime				__attribute__ ((packed));
+   unsigned long cstime				__attribute__ ((packed));
+   unsigned long cnswap				__attribute__ ((packed));
+   unsigned int  processor			__attribute__ ((packed));
    unsigned char flag				__attribute__ ((packed));
    unsigned char pad[3]				__attribute__ ((packed));
-   /* See IMPORTANT NOTE above */
 };
 
 #define PID_STATS_SIZE	(sizeof(struct pid_stats))
@@ -337,9 +390,12 @@ struct pid_stats {
 struct stats_serial {
    unsigned int  rx				__attribute__ ((aligned (8)));
    unsigned int  tx				__attribute__ ((packed));
+   unsigned int  frame				__attribute__ ((packed));
+   unsigned int  parity				__attribute__ ((packed));
+   unsigned int  brk				__attribute__ ((packed));
+   unsigned int  overrun			__attribute__ ((packed));
    unsigned int  line				__attribute__ ((packed));
    unsigned char pad[4]				__attribute__ ((packed));
-   /* See IMPORTANT NOTE above */
 };
 
 #define STATS_SERIAL_SIZE	(sizeof(struct stats_serial))
@@ -362,29 +418,42 @@ struct stats_net_dev {
    unsigned long tx_fifo_errors			__attribute__ ((aligned (8)));
    unsigned long rx_frame_errors		__attribute__ ((aligned (8)));
    unsigned long tx_carrier_errors		__attribute__ ((aligned (8)));
-   unsigned char interface[MAX_IFACE_LEN]	__attribute__ ((aligned (8)));
-   /* See IMPORTANT NOTE above */
+   char		 interface[MAX_IFACE_LEN]	__attribute__ ((aligned (8)));
 };
 
 #define STATS_NET_DEV_SIZE	(sizeof(struct stats_net_dev))
 
+
+/*
+ * stats_irq_cpu->irq:       IRQ#-A
+ * stats_irq_cpu->interrupt: number of IRQ#-A for proc 0
+ * stats_irq_cpu->irq:       IRQ#-B
+ * stats_irq_cpu->interrupt: number of IRQ#-B for proc 0
+ * ...
+ * stats_irq_cpu->irq:       (undef'd)
+ * stats_irq_cpu->interrupt: number of IRQ#-A for proc 1
+ * stats_irq_cpu->irq:       (undef'd)
+ * stats_irq_cpu->interrupt: number of IRQ#-B for proc 1
+ * ...
+ */
 struct stats_irq_cpu {
    unsigned int interrupt			__attribute__ ((aligned (8)));
    unsigned int irq				__attribute__ ((packed));
-   /* See IMPORTANT NOTE above */
 };
 
 #define STATS_IRQ_CPU_SIZE	(sizeof(struct stats_irq_cpu))
 #define STATS_ONE_IRQ_SIZE	(sizeof(int) * NR_IRQS)
 
 struct disk_stats {
+   unsigned long long rd_sect			__attribute__ ((aligned (16)));
+   unsigned long long wr_sect			__attribute__ ((aligned (16)));
+   unsigned long rd_ticks			__attribute__ ((aligned (16)));
+   unsigned long wr_ticks			__attribute__ ((aligned (8)));
+   unsigned long tot_ticks			__attribute__ ((aligned (8)));
+   unsigned long rq_ticks			__attribute__ ((aligned (8)));
+   unsigned long nr_ios				__attribute__ ((aligned (8)));
    unsigned int  major				__attribute__ ((aligned (8)));
-   unsigned int  index				__attribute__ ((packed));
-   unsigned int  nr_ios				__attribute__ ((packed));
-   unsigned int  rd_sect			__attribute__ ((packed));
-   unsigned int  wr_sect			__attribute__ ((packed));
-   unsigned char pad[4]				__attribute__ ((packed));
-   /* See IMPORTANT NOTE above */
+   unsigned int  minor				__attribute__ ((packed));
 };
 
 #define DISK_STATS_SIZE		(sizeof(struct disk_stats))
@@ -428,10 +497,65 @@ struct tstamp {
 #define DEF_TMSTART	"08:00:00"
 #define DEF_TMEND	"18:00:00"
 
+
 /* Using 'do ... while' makes this macro safer to use (trailing semicolon) */
-#define CLOSE_ALL(_fd_)		do { \
+#define CLOSE_ALL(_fd_)		do {		\
 				close(_fd_[0]); \
 				close(_fd_[1]); \
 				} while (0)
 
+#define CLOSE(_fd_)		if (_fd_ >= 0)	\
+				close(_fd_)
+
+#define SREALLOC(S, TYPE, SIZE)	do {							\
+   				   TYPE *p;						\
+				   p = S;						\
+   				   if ((S = (TYPE *) realloc(S, (SIZE))) == NULL) {	\
+				      perror("realloc");				\
+				      exit(4);						\
+				   }							\
+   				   if (!p)						\
+      				      memset(S, 0, (SIZE));				\
+				} while (0)
+
+/* Functions */
+extern int	    check_disk_reg(struct file_hdr *, struct disk_stats * [],
+				   short, short, int);
+extern unsigned int check_iface_reg(struct file_hdr *,
+				    struct stats_net_dev * [],
+				    short, short, unsigned int);
+extern int	    datecmp(struct tm *, struct tstamp *);
+unsigned long long  get_per_cpu_interval(struct stats_one_cpu *,
+					 struct stats_one_cpu *);
+extern char	   *get_devname(unsigned int, unsigned int, int);
+extern void	    init_bitmap(unsigned char [], unsigned char, unsigned int);
+extern void	    init_stats(struct file_stats [], unsigned int [][NR_IRQS]);
+extern int	    next_slice(unsigned long long, unsigned long long,
+			       struct file_hdr *, int, long);
+extern int	    parse_sar_opt(char * [], int, unsigned int *,
+				  unsigned int *, short *, int,
+				  unsigned char [], unsigned char []);
+extern int	    parse_sar_I_opt(char * [], int *, unsigned int *, short *,
+				    unsigned char []);
+extern int	    parse_sa_P_opt(char * [], int *, unsigned int *, short *,
+				   unsigned char []);
+extern int	    parse_sar_n_opt(char * [], int *, unsigned int *, short *);
+extern int	    parse_timestamp(char * [], int *, struct tstamp *,
+				    const char *);
+extern void	    prep_file_for_reading(int *, char *, struct file_hdr *,
+					  unsigned int *, unsigned int);
+extern void	    get_itv_value(struct file_stats *, struct file_stats *,
+				  unsigned int, unsigned long long *,
+				  unsigned long long *);
+extern void	    print_report_hdr(unsigned short, unsigned int,
+				     struct tm *, struct file_hdr *);
+extern int	    sa_fread(int, void *, int, int);
+extern void	    salloc_cpu_array(struct stats_one_cpu * [], unsigned int);
+extern void	    salloc_disk_array(struct disk_stats * [], int);
+extern void	    salloc_irqcpu_array(struct stats_irq_cpu * [],
+					unsigned int, unsigned int);
+extern void	    salloc_net_dev_array(struct stats_net_dev * [],
+					 unsigned int);
+extern void	    salloc_serial_array(struct stats_serial * [], int);
+
 #endif  /* _SA_H */
