@@ -1,6 +1,6 @@
 /*
  * sar and sadf common routines.
- * (C) 1999-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -182,15 +182,18 @@ int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
 {
    unsigned long file_interval, entry;
    static unsigned long long last_uptime = 0;
-   int min, max, pt1, pt2;
+   int min, max, pt1, pt2, cpu_nr;
    double f;
 
    if (!last_uptime || reset)
       last_uptime = uptime_ref;
 
+   cpu_nr = file_hdr->sa_proc;
+   if (!cpu_nr)
+      cpu_nr = 1;
+
    /* Interval cannot be greater than 0xffffffff here */
-   f = (((double) ((uptime - last_uptime) & 0xffffffff)) /
-	(file_hdr->sa_proc + 1)) / HZ;
+   f = (((double) ((uptime - last_uptime) & 0xffffffff)) / cpu_nr) / HZ;
    file_interval = (unsigned long) f;
    if ((f * 10) - (file_interval * 10) >= 5)
       file_interval++; /* Rounding to correct value */
@@ -210,8 +213,7 @@ int next_slice(unsigned long long uptime_ref, unsigned long long uptime,
     *       (Pn * Iu) or (P'n * Iu) belongs to In
     * with  Pn = En / Iu and P'n = En / Iu + 1
     */
-   f = (((double) ((uptime - uptime_ref) & 0xffffffff)) /
-	(file_hdr->sa_proc + 1)) / HZ;
+   f = (((double) ((uptime - uptime_ref) & 0xffffffff)) / cpu_nr) / HZ;
    entry = (unsigned long) f;
    if ((f * 10) - (entry * 10) >= 5)
       entry++;
@@ -312,7 +314,7 @@ void get_itv_value(struct file_stats *file_stats_curr,
    if (!(*g_itv))	/* Paranoia checking */
       *g_itv = 1;
 
-   if (nr_proc) {
+   if (nr_proc > 1) {
       if (!file_stats_prev->uptime0)
 	 *itv = file_stats_curr->uptime0;
       else
@@ -542,10 +544,10 @@ unsigned long long get_per_cpu_interval(struct stats_one_cpu *st_cpu_i,
 {
    return ((st_cpu_i->per_cpu_user + st_cpu_i->per_cpu_nice +
 	    st_cpu_i->per_cpu_system + st_cpu_i->per_cpu_iowait +
-	    st_cpu_i->per_cpu_idle) -
+	    st_cpu_i->per_cpu_idle + st_cpu_i->per_cpu_steal) -
 	   (st_cpu_j->per_cpu_user + st_cpu_j->per_cpu_nice +
 	    st_cpu_j->per_cpu_system + st_cpu_j->per_cpu_iowait +
-	    st_cpu_j->per_cpu_idle));
+	    st_cpu_j->per_cpu_idle + st_cpu_j->per_cpu_steal));
 }
 
 
@@ -738,7 +740,7 @@ int parse_sar_n_opt(char *argv[], int *opt, unsigned int *actflag,
       *actflag |= A_NET_NFS;
    else if (!strcmp(argv[*opt], K_NFSD))
       *actflag |= A_NET_NFSD;
-   else if (!strcmp(argv[*opt], K_FULL)) {
+   else if (!strcmp(argv[*opt], K_ALL)) {
       *actflag |= A_NET_DEV + A_NET_EDEV + A_NET_SOCK + A_NET_NFS + A_NET_NFSD;
       *dis_hdr = 9;
    }
