@@ -1,6 +1,6 @@
 /*
  * sadc: system activity data collector
- * (C) 1999-2005 by Sebastien GODARD (sysstat <at> wanadoo.fr)
+ * (C) 1999-2006 by Sebastien GODARD (sysstat <at> wanadoo.fr)
  *
  ***************************************************************************
  * This program is free software; you can redistribute it and/or modify it *
@@ -33,7 +33,6 @@
 #include <sys/stat.h>
 #include <sys/utsname.h>
 
-#include "version.h"
 #include "sa.h"
 #include "common.h"
 #include "ioconf.h"
@@ -47,10 +46,10 @@
 #define _(string) (string)
 #endif
 
-/* Nb of processors on the machine. A value of 1 means two processors */
-int cpu_nr = -1;
+/* Nb of processors on the machine */
+int cpu_nr = 0;
 unsigned int serial_nr = 0, iface_nr = 0, irqcpu_nr = 0, disk_nr = 0;
-int pid_nr = 0;
+int pid_idx = 0, pid_nr = 0;
 unsigned int sadc_actflag;
 long interval = 0;
 int kb_shift = 0;
@@ -81,12 +80,10 @@ void usage(char *progname)
     * Don't show options like -x ALL or -X SELF.
     * They should only be used with sar.
     */
-   fprintf(stderr, _("sysstat version %s\n"
-		   "(C) Sebastien Godard\n"
-	           "Usage: %s [ options... ] [ <interval> [ <count> ] ] [ <outfile> ]\n"
+   fprintf(stderr, _("Usage: %s [ options... ] [ <interval> [ <count> ] ] [ <outfile> ]\n"
 		   "Options are:\n"
 		   "[ -d ] [ -F ] [ -I ] [ -V ]\n"),
-	   VERSION, progname);
+	   progname);
    exit(1);
 }
 
@@ -162,11 +159,11 @@ int save_pid(unsigned long pid)
 {
    int i = 0;
 
-   while ((i < pid_nr) && (all_pids[i] != pid))
+   while ((i < pid_idx) && (all_pids[i] != pid))
       i++;
    /* PID not found: insert it if possible */
-   if ((i == pid_nr) && (pid_nr < MAX_PID_NR))
-      all_pids[pid_nr++] = pid;
+   if ((i == pid_idx) && (pid_idx < MAX_PID_NR))
+      all_pids[pid_idx++] = pid;
 
    return i;
 }
@@ -216,7 +213,7 @@ void get_pid_list(void)
    /* Get directory entries */
    while ((drp = readdir(dir)) != NULL) {
       if (isdigit(drp->d_name[0])) {
-	 /* Save PID in th list */
+	 /* Save PID in the list */
 	 if (save_pid(atol(drp->d_name)) == MAX_PID_NR)
 	    break;
       }
@@ -329,7 +326,7 @@ void sa_sys_init(unsigned int *flags)
 {
    /* How many processors on this machine ? */
    if ((cpu_nr = get_cpu_nr(NR_CPUS)) > 0)
-      SREALLOC(st_cpu, struct stats_one_cpu, STATS_ONE_CPU_SIZE * (cpu_nr + 1));
+      SREALLOC(st_cpu, struct stats_one_cpu, STATS_ONE_CPU_SIZE * cpu_nr);
 
    /* Get serial lines that support accounting */
    if ((serial_nr = get_serial_lines())) {
@@ -337,10 +334,10 @@ void sa_sys_init(unsigned int *flags)
       SREALLOC(st_serial, struct stats_serial, STATS_SERIAL_SIZE * serial_nr);
    }
    /* Get number of interrupts available per processor */
-   if (cpu_nr > 0) {
+   if (cpu_nr) {
       if ((irqcpu_nr = get_irqcpu_nb(NR_IRQS)))
 	 SREALLOC(st_irq_cpu, struct stats_irq_cpu,
-		  STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr);
+		  STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr);
    }
    else
       /* IRQ per processor are not provided by sadc on UP machines */
@@ -427,16 +424,17 @@ void setup_file_hdr(int fd, size_t *file_stats_size)
 
    /* Ok, now fill the header */
    file_hdr.sa_actflag = sadc_actflag;
-   file_hdr.sa_magic   = SA_MAGIC;
+   file_hdr.sa_magic = SA_MAGIC;
    file_hdr.sa_st_size = FILE_STATS_SIZE;
-   file_hdr.sa_day     = loc_time.tm_mday;
-   file_hdr.sa_month   = loc_time.tm_mon;
-   file_hdr.sa_year    = loc_time.tm_year;
-   file_hdr.sa_proc    = cpu_nr;
-   file_hdr.sa_nr_pid  = pid_nr;
-   file_hdr.sa_serial  = serial_nr;
-   file_hdr.sa_irqcpu  = irqcpu_nr;
-   file_hdr.sa_iface   = iface_nr;
+   file_hdr.sa_day = loc_time.tm_mday;
+   file_hdr.sa_month = loc_time.tm_mon;
+   file_hdr.sa_year = loc_time.tm_year;
+   file_hdr.sa_sizeof_long = sizeof(long);
+   file_hdr.sa_proc = cpu_nr;
+   file_hdr.sa_nr_pid = pid_nr;
+   file_hdr.sa_serial = serial_nr;
+   file_hdr.sa_irqcpu = irqcpu_nr;
+   file_hdr.sa_iface = iface_nr;
    if (GET_DISK(sadc_actflag))
       file_hdr.sa_nr_disk = disk_nr;
    else
@@ -501,12 +499,13 @@ void write_dummy_record(int ofd, size_t file_stats_size, unsigned int *flags)
  ***************************************************************************
  * Write stats.
  * NB: sadc provides all the stats, including:
- * -> CPU utilization per processor (on SMP machines only)
- * -> IRQ per processor (on SMP machines only)
+ * -> CPU utilization per processor (*)
+ * -> IRQ per processor (*)
  * -> number of each IRQ (if -I option passed to sadc), including APIC
  *    interrupts sources
  * -> device stats for sar -d (kernels 2.4 and newer only, and only if
  *    -d option passed to sadc)
+ * (*): on SMP machines only, even if there is only one available proc.
  ***************************************************************************
  */
 void write_stats(int ofd, size_t file_stats_size, unsigned int *flags)
@@ -521,8 +520,8 @@ void write_stats(int ofd, size_t file_stats_size, unsigned int *flags)
    }
    if ((nb = write(ofd, &file_stats, file_stats_size)) != file_stats_size)
       p_write_error();
-   if (cpu_nr > 0) {
-      if ((nb = write(ofd, st_cpu, STATS_ONE_CPU_SIZE * (cpu_nr + 1))) != (STATS_ONE_CPU_SIZE * (cpu_nr + 1)))
+   if (cpu_nr) {
+      if ((nb = write(ofd, st_cpu, STATS_ONE_CPU_SIZE * cpu_nr)) != (STATS_ONE_CPU_SIZE * cpu_nr))
 	 p_write_error();
    }
    if (GET_ONE_IRQ(sadc_actflag)) {
@@ -539,8 +538,8 @@ void write_stats(int ofd, size_t file_stats_size, unsigned int *flags)
 	 p_write_error();
    }
    if (irqcpu_nr) {
-      if ((nb = write(ofd, st_irq_cpu, STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr))
-	  != (STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr))
+      if ((nb = write(ofd, st_irq_cpu, STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr))
+	  != (STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr))
 	 p_write_error();
    }
    if (iface_nr) {
@@ -676,7 +675,7 @@ void open_ofile(int *ofd, char ofile[], size_t *file_stats_size, unsigned int *f
 	 if (file_hdr.sa_irqcpu != irqcpu_nr) {
 	    irqcpu_nr = file_hdr.sa_irqcpu;
 	    SREALLOC(st_irq_cpu, struct stats_irq_cpu,
-		     STATS_IRQ_CPU_SIZE * (cpu_nr + 1) * irqcpu_nr);
+		     STATS_IRQ_CPU_SIZE * cpu_nr * irqcpu_nr);
 	 }
 	 if (file_hdr.sa_nr_disk != disk_nr) {
 	    if (!file_hdr.sa_nr_disk)
@@ -704,7 +703,7 @@ void read_proc_stat(unsigned int flags)
    struct disk_stats *st_disk_i;
    static char line[8192];
    unsigned long long cc_user, cc_nice, cc_system, cc_hardirq, cc_softirq;
-   unsigned long long cc_idle, cc_iowait;
+   unsigned long long cc_idle, cc_iowait, cc_steal;
    unsigned int v_tmp[5], v_major, v_index;
    int proc_nb, i, pos;
 
@@ -722,15 +721,17 @@ void read_proc_stat(unsigned int flags)
 	  * to one processor to avoid rounding problems.
 	  */
 	 file_stats.cpu_iowait = 0;	/* For pre 2.5 kernels */
+	 file_stats.cpu_steal = 0;
 	 cc_hardirq = cc_softirq = 0;
-	 sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu",
+	 sscanf(line + 5, "%llu %llu %llu %llu %llu %llu %llu %llu",
 		&(file_stats.cpu_user),   &(file_stats.cpu_nice),
 		&(file_stats.cpu_system), &(file_stats.cpu_idle),
-		&(file_stats.cpu_iowait), &cc_hardirq, &cc_softirq);
+		&(file_stats.cpu_iowait), &cc_hardirq, &cc_softirq,
+		&(file_stats.cpu_steal));
 
 	 /*
 	  * Time spent in system mode also includes time spent
-	  * servicing interrrupts and softirqs
+	  * servicing hard interrrupts and softirqs.
 	  */
 	 file_stats.cpu_system += cc_hardirq + cc_softirq;
 	
@@ -746,33 +747,32 @@ void read_proc_stat(unsigned int flags)
 	  */
 	 file_stats.uptime = file_stats.cpu_user   + file_stats.cpu_nice +
 	                     file_stats.cpu_system + file_stats.cpu_idle +
-	                     file_stats.cpu_iowait;
+	                     file_stats.cpu_iowait + file_stats.cpu_steal;
       }
 
       else if (!strncmp(line, "cpu", 3)) {
-	 if (cpu_nr > 0) {
+	 if (cpu_nr) {
 	    /*
 	     * Read the number of jiffies spent in the different modes
 	     * (user, nice, etc) for current proc.
 	     * This is done only on SMP machines.
-	     * Warning: st_cpu_i struct is _not_ allocated even if the kernel
-	     * has SMP support enabled.
 	     */
-	    cc_iowait = 0;	/* For pre 2.5 kernels */
+	    cc_iowait = cc_steal = 0;	/* For pre 2.5 kernels */
 	    cc_hardirq = cc_softirq = 0;
-	    sscanf(line + 3, "%d %llu %llu %llu %llu %llu %llu %llu",
+	    sscanf(line + 3, "%d %llu %llu %llu %llu %llu %llu %llu %llu",
 		   &proc_nb,
 		   &cc_user, &cc_nice, &cc_system, &cc_idle, &cc_iowait,
-		   &cc_hardirq, &cc_softirq);
+		   &cc_hardirq, &cc_softirq, &cc_steal);
 	    cc_system += cc_hardirq + cc_softirq;
 	
-	    if (proc_nb <= cpu_nr) {
+	    if (proc_nb < cpu_nr) {
 	       st_cpu_i = st_cpu + proc_nb;
 	       st_cpu_i->per_cpu_user   = cc_user;
 	       st_cpu_i->per_cpu_nice   = cc_nice;
 	       st_cpu_i->per_cpu_system = cc_system;
 	       st_cpu_i->per_cpu_idle   = cc_idle;
 	       st_cpu_i->per_cpu_iowait = cc_iowait;
+	       st_cpu_i->per_cpu_steal  = cc_steal;
 	    }
 	    /* else:
 	     * Additional CPUs have been dynamically registered in /proc/stat.
@@ -782,7 +782,7 @@ void read_proc_stat(unsigned int flags)
 	    if (!proc_nb)
 	       /* Compute uptime reduced to one proc using proc#0 */
 	       file_stats.uptime0 = cc_user + cc_nice + cc_system +
-	       			    cc_idle + cc_iowait;
+	       			    cc_idle + cc_iowait + cc_steal;
 	 }
       }
 
@@ -1037,13 +1037,16 @@ void read_proc_vmstat(void)
  * Read stats from /proc/<pid>/stat
  ***************************************************************************
  */
-void read_pid_stat(void)
+void read_pid_stat(unsigned int flags)
 {
    FILE *fp;
    int pid;
    static char filename[24];
 
-   for (pid = 0; pid < pid_nr; pid++) {
+   if (WANT_ALL_PIDS(flags))
+      get_pid_list();
+
+   for (pid = 0; pid < pid_idx; pid++) {
 
       if (!all_pids[pid])
 	 continue;
@@ -1152,7 +1155,7 @@ void read_interrupts_stat(void)
 	    p = st_irq_cpu + irq;
 	    sscanf(line, "%3u", &(p->irq));
 	
-	    for (cpu = 0; cpu <= cpu_nr; cpu++) {
+	    for (cpu = 0; cpu < cpu_nr; cpu++) {
 	       p = st_irq_cpu + cpu * irqcpu_nr + irq;
 	       /*
 		* No need to set (st_irq_cpu + cpu * irqcpu_nr)->irq:
@@ -1386,9 +1389,9 @@ void read_net_nfs_stat(void)
 		&(file_stats.nfs_rpccnt), &(file_stats.nfs_rpcretrans));
 	
       else if (!strncmp(line, "proc3", 5))
-	 sscanf(line + 5, "%*u %u %*u %*u %u %*u %u %u",
+	 sscanf(line + 5, "%*u %*u %u %*u %*u %u %*u %u %u",
 		&(file_stats.nfs_getattcnt), &(file_stats.nfs_accesscnt),
-		&(file_stats.nfs_writecnt), &(file_stats.nfs_readcnt));
+		&(file_stats.nfs_readcnt), &(file_stats.nfs_writecnt));
    }
 
    fclose(fp);
@@ -1424,9 +1427,9 @@ void read_net_nfsd_stat(void)
 		&(file_stats.nfsd_rpccnt), &(file_stats.nfsd_rpcbad));
 	
       else if (!strncmp(line, "proc3", 5))
-	 sscanf(line + 5, "%*u %u %*u %*u %u %*u %u %u",
+	 sscanf(line + 5, "%*u %*u %u %*u %*u %u %*u %u %u",
 		&(file_stats.nfsd_getattcnt), &(file_stats.nfsd_accesscnt),
-		&(file_stats.nfsd_writecnt), &(file_stats.nfsd_readcnt));
+		&(file_stats.nfsd_readcnt), &(file_stats.nfsd_writecnt));
    }
 
    fclose(fp);
@@ -1584,7 +1587,7 @@ void read_stats(unsigned int *flags)
    /* else disks are in /proc/stat */
 
    if (pid_nr)
-       read_pid_stat();
+       read_pid_stat(*flags);
     if (serial_nr)
       read_serial_stat();
    if (irqcpu_nr)
@@ -1754,7 +1757,7 @@ int main(int argc, char **argv)
 	 flags |= S_F_L_OPTION;
 
       else if (!strcmp(argv[opt], "-V"))
-	 usage(argv[0]);
+	 print_version();
 
       else if (!strcmp(argv[opt], "-z"))	/* Set by sar command */
 	 optz = 1;
@@ -1768,8 +1771,7 @@ int main(int argc, char **argv)
 	 sadc_actflag |= A_PID;
 	 if (argv[++opt]) {
 	    if (!strcmp(argv[opt], K_ALL)) {
-	       /* Get PID list */
-	       get_pid_list();
+	       flags |= S_F_X_ALL;
 	       set_pflag(strcmp(argv[opt - 1], "-x"), 0);
 	       continue;	/* Next option */
 	    }
@@ -1837,13 +1839,19 @@ int main(int argc, char **argv)
 
    /* -x and -X options ignored when writing to a file */
    if (ofile[0]) {
-      pid_nr = 0;
+      pid_idx = 0;
+      flags &= ~S_F_X_ALL;
       sadc_actflag &= ~A_PID;
    }
    else
       /* -L option ignored when writing to STDOUT */
       flags &= ~S_F_L_OPTION;
 
+   if (WANT_ALL_PIDS(flags))
+      pid_nr = MAX_PID_NR;
+   else
+      pid_nr = pid_idx;
+
    /* Init structures according to machine architecture */
    sa_sys_init(&flags);
 
